searchData={"items":[{"type":"module","title":"Boing","doc":"A key-based debouncer.\n\nActions are debounced by specifying a key for related actions. When the first action for that key\noccurs, a 1 second (by default, but configurable) timer is started. The timer is restarted for each\nsubsequent request to the same key that occurs before the timer expires. When the timer expires,\nthe most recent request is permitted, and the previous requests are debounced.","ref":"Boing.html"},{"type":"module","title":"Implementation details - Boing","doc":"Internally, this module uses `Registry` and `DynamicSupervisor` to start a unique registered\n`m::gen_statem` for each key with an active timer. Requests make a blocking call to that process,\nwhich when the timer expires returns `:ok` to the most recent request, and \n`{:error, debounced: key}` to all others.\n\nProcesses are short-lived and de-register themselves once the timer expires.\n\nThe keys used are associated with a `Registry` that is started as part of Boing's supervision tree,\nand can be considered global. All calls made to `Boing.debounce/2` for the same key within the\ntimeout period are made to the same `m::gen_statem` process, which will block and return\nwhen the timer expires.","ref":"Boing.html#module-implementation-details"},{"type":"function","title":"Boing.debounce/2","doc":"Debounce requests for the specified key.\n\n`timeout` is the number of milliseconds to wait before allowing the request. Defaults to 1000ms.\n\nInitially, no timer is active. When this function is called for a `key` which has no active timer,\na new timer is started for `timeout` milliseconds (1 second by default). Any subsequent calls while\nthe timer is active restart the timer. When the timer expires, `:ok` is returned to the most\nrecent call, and `{:error, debounced: key}` to all the other calls that were made while the timer\nwas active.","ref":"Boing.html#debounce/2"},{"type":"module","title":"Boing.ReqPlugin","doc":"Req plugin that provides debouncing of GET requests.","ref":"Boing.ReqPlugin.html"},{"type":"module","title":"Usage - Boing.ReqPlugin","doc":"1. Configure Req to use the plugin:\n   ```elixir\n   req = Req.new() |> Boing.ReqPlugin.attach()\n   ```\n2. Concurrent requests to the same URL are debounced:\n   ```elixir\n   for _ <- 1..3, do: spawn(fn -> dbg(Req.get(client, url: \"http://example.com\")) end)\n   #=> [#PID<0.208.0>, #PID<0.209.0>, #PID<0.210.0>]\n   Req.get(client, url: \"http://example.com\") #=> {:error, %Boing.ReqPlugin.DebouncedException{url: \"http://example.com\"}}\n   Req.get(client, url: \"http://example.com\") #=> {:error, %Boing.ReqPlugin.DebouncedException{url: \"http://example.com\"}}\n   Req.get(client, url: \"http://example.com\") #=> {:ok, %Req.Response{body: ...}}\n   ```","ref":"Boing.ReqPlugin.html#module-usage"},{"type":"module","title":"Custom timeout - Boing.ReqPlugin","doc":"The `:debounce_timeout` option can be supplied to customize the timeout, which defaults to 1000ms.\n\n    client = Req.new() |> Boing.ReqPlugin.attach(debounce_timeout: 500)\n    for _ <- 1..2, do: Process.sleep(600) && spawn(fn -> dbg(Req.get(client, url: \"http://example.com\")) end)\n    #=> {:ok, %Req.Response{}}\n    #=> {:ok, %Req.Response{}}\n\nIn this example, both requests return `{:ok, %Req.Response{...}}` and are not debounced, due to\nthe shorter timeout.","ref":"Boing.ReqPlugin.html#module-custom-timeout"},{"type":"function","title":"Boing.ReqPlugin.attach/2","doc":"Attach the plugin to the Req client.\n\n    req = Req.new() |> Boing.ReqPlugin.attach()","ref":"Boing.ReqPlugin.html#attach/2"},{"type":"module","title":"Boing.TeslaMiddleware","doc":"Middleware for Tesla that provides debouncing of GET requests.","ref":"Boing.TeslaMiddleware.html"},{"type":"module","title":"Usage - Boing.TeslaMiddleware","doc":"1. Configure Tesla to use the debouncer middleware:\n   ```elixir\n   client = Tesla.client([Boing.TeslaMiddleware])\n   ```\n2. Concurrent requests to the same URL are debounced:\n   ```elixir\n   for _ <- 1..3, do: spawn(fn -> dbg(Tesla.get(client, \"http://example.com\")) end)\n   #=> [#PID<0.208.0>, #PID<0.209.0>, #PID<0.210.0>]\n   Tesla.get(client, \"http://example.com\") #=> {:error, [debounced: \"http://example.com\"]}\n   Tesla.get(client, \"http://example.com\") #=> {:error, [debounced: \"http://example.com\"]}\n   Tesla.get(client, \"http://example.com\") #=> {:ok, %Tesla.Env{body: ...}}\n   ```","ref":"Boing.TeslaMiddleware.html#module-usage"},{"type":"module","title":"Custom timeout - Boing.TeslaMiddleware","doc":"The `:debounce_timeout` option can be supplied to customize the timeout, which defaults to 1000ms.\n\n    client = Tesla.client([{Boing.TeslaMiddleware, debounce_timeout: 500}])\n    for _ <- 1..2, do: Process.sleep(600) && spawn(fn -> dbg(Tesla.get!(client, \"http://example.com\")) end)\n    #=> {:ok, %Tesla.Env{...}}\n    #=> {:ok, %Tesla.Env{...}}\n\nIn this example, both requests return `{:ok, %Tesla.Env{...}}` and are not debounced, due to the\nshorter timeout.","ref":"Boing.TeslaMiddleware.html#module-custom-timeout"},{"type":"exception","title":"Boing.ReqPlugin.DebouncedException","doc":"","ref":"Boing.ReqPlugin.DebouncedException.html"},{"type":"extras","title":"Boing","doc":"# Boing\n\nA key-based debouncer with built-in implementations for `Tesla` and `Req` HTTP clients.\n\nFor HTTP requests, GET requests to the same URL are debounced, all other requests are unaffected.\nDebounced requests are rejected client-side and the HTTP request is not made.","ref":"readme.html"},{"type":"extras","title":"Example - Boing","doc":"","ref":"readme.html#example"},{"type":"extras","title":"Tesla - Boing","doc":"```elixir\niex(1)> client = Tesla.client([Boing.TeslaMiddleware])\n%Tesla.Client{...}\niex(2)> for _ <- 1..3, do: spawn(fn -> dbg(Tesla.get(client, \"http://example.com\")) end)\n[#PID<0.208.0>, #PID<0.209.0>, #PID<0.210.0>]\nTesla.get(client, \"http://example.com\") #=> {:error, [debounced: \"http://example.com\"]}\nTesla.get(client, \"http://example.com\") #=> {:error, [debounced: \"http://example.com\"]}\nTesla.get(client, \"http://example.com\") #=> {:ok, %Tesla.Env{body: ...}}\n```\n\nFor full documentation, see `Boing.TeslaMiddleware`.","ref":"readme.html#tesla"},{"type":"extras","title":"Req - Boing","doc":"```elixir\niex(1)> client = Req.new() |> Boing.ReqPlugin.attach()\n%Req.Request{...}\niex(2)> for _ <- 1..3, do: spawn(fn -> dbg(Req.get(client, url: \"http://example.com\")) end)\n[#PID<0.208.0>, #PID<0.209.0>, #PID<0.210.0>]\nReq.get(client, url: \"http://example.com\") #=> {:error, %Boing.ReqPlugin.DebouncedException{url: \"http://example.com\"}}\nReq.get(client, url: \"http://example.com\") #=> {:error, %Boing.ReqPlugin.DebouncedException{url: \"http://example.com\"}}\nReq.get(client, url: \"http://example.com\") #=> {:ok, %Req.Response{body: ...}}\n```\n\nFor full documentation see `Boing.ReqPlugin`.","ref":"readme.html#req"},{"type":"extras","title":"Direct - Boing","doc":"```elixir\niex(1)> for i <- 1..3, do: spawn(fn -> IO.puts(\"Foo #{i}: #{inspect(Boing.debounce(\"foo\"))}\") end)\n[#PID<0.221.0>, #PID<0.222.0>, #PID<0.223.0>]\niex(2)> for i <- 1..3, do: spawn(fn -> IO.puts(\"Bar #{i}: #{inspect(Boing.debounce(\"bar\"))}\") end)\n[#PID<0.225.0>, #PID<0.226.0>, #PID<0.227.0>]\nBar 3: :ok\nFoo 3: :ok\nBar 1: {:error, [debounced: \"bar\"]}\nFoo 1: {:error, [debounced: \"foo\"]}\nFoo 2: {:error, [debounced: \"foo\"]}\nBar 2: {:error, [debounced: \"bar\"]}\n```\n\nFor full documentation see `Boing`.","ref":"readme.html#direct"},{"type":"extras","title":"Overview - Boing","doc":"Actions are debounced by assigning a key to related actions. For the Req and Tesla modules, the\nGET request URL is automatically used as the key. When the first action for that key occurs,\na 1 second (by default, but configurable) timer is started. The timer is restarted for each subsequent\nrequest to the same key/URL that occurs before the timer expires. When the timer eventually expires,\nthe most recent request is permitted, and the previous requests are debounced.\n\nWhile this module includes support for Tesla and Req, the core logic is implementation-independent\nand can be used for any purpose.\n\nFor full documentation of the underlying implementation, see `Boing`.","ref":"readme.html#overview"},{"type":"extras","title":"Installation - Boing","doc":"Boing can be installed by adding it to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:boing, \"~> 0.1.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Documentation - Boing","doc":"Full documentation is available in `iex` via the [`h`](`IEx.Helpers.h/1`) helper and also is published\nto [Hex](https://hexdocs.pm/boing/).","ref":"readme.html#documentation"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.2"}}